#%%
"""
Plot average spectra from 31 Aug based on circle masks (Fig. 1 c)
"""

import hsi_functions as hsi
import hsi_shoot_locs as slocs
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib import ticker
import seaborn as sns
from matplotlib.patches import Patch
import numpy as np
from fig_colors import treatment_palette as c_palette 
from data_directory_path import data_directory
import os
os.chdir(data_directory)

# Save plot? 
save = True

def circle_av(im, center, radius):
    """
    Function to get average value over a circle with set radius and center
    """
    h = 1024
    w = 1024
    Y, X = np.ogrid[:h, :w]
    dist_from_center = np.sqrt((X - center[0])**2 + (Y-center[1])**2)

    mask = dist_from_center >= radius

    num_bands = im.shape[0]
    avg_values = np.zeros(num_bands)

    for band in range(num_bands):
        band_slice = im[band]
        mask_arr = np.ma.masked_array(band_slice, mask=mask)
        avg_values[band] = np.nanmean(mask_arr)

    return avg_values

#%%
im = hsi.read_img('./data/HSI/HSI_output/refl_HSI_data/20220831_refl-allIT__rep2')
wav, fwhm = hsi.get_wav('./data/HSI/HSI_output/refl_HSI_data/20220831_refl-allIT__rep2')

shoot_locs = slocs.sloc_dict['20220831']

pots = [54, 25, 10, 28, 9, 6, 55, 30]
spectra = {}
spectra['wav'] = wav

for i, pot in zip(shoot_locs, pots):
    spectra[pot] = circle_av(im, i, 50)

spec_df = pd.DataFrame.from_dict(spectra)
spec_df.loc[(spec_df['wav'] > 600) & (spec_df['wav'] < 625)] = np.nan
spec_df.loc[(spec_df['wav'] >= 635.7) & (spec_df['wav'] <= 	657.1)] = np.nan
spec_df = spec_df.dropna()


SMALL_SIZE = 12
MEDIUM_SIZE = 14
BIGGER_SIZE = 16
plt.rc('font', size=MEDIUM_SIZE) 
plt.rc('axes', titlesize=MEDIUM_SIZE)  
plt.rc('axes', labelsize=MEDIUM_SIZE) 
plt.rc('xtick', labelsize=SMALL_SIZE)   
plt.rc('ytick', labelsize=SMALL_SIZE)    
plt.rc('legend', fontsize=SMALL_SIZE)    

spectra_df = pd.DataFrame.from_dict(spectra)
df = spectra_df.melt(id_vars='wav')
df['treatment'] = np.where(((df['variable'] == 54) | (df['variable'] == 55) | (df['variable'] == 10) | (df['variable'] == 9)), 'D', 'C')
df.loc[(df['wav'] > 600) & (df['wav'] < 625)] = np.nan
df.loc[(df['wav'] >= 635.7) & (df['wav'] <= 	657.1)] = np.nan

fig, ax = plt.subplots(figsize=(4,4))

pri_c = '#b4e2ba'
rep_c = '#fece65'

plt.axvspan(523.5, 538.5, alpha=0.5, facecolor=pri_c, edgecolor = 'None') # R531
plt.axvspan(558.3, 570.3, alpha=0.5, facecolor=pri_c, edgecolor = 'None') # R564
plt.axvspan(694, 706, alpha=0.5, facecolor=rep_c, edgecolor = 'None') # R531
plt.axvspan(736.4, 749, alpha=0.5, facecolor=rep_c, edgecolor = 'None') # R564
plt.axvspan(769.6, 787.6, alpha=0.5, facecolor=rep_c, edgecolor = 'None') # R531
plt.axvspan(664.5, 678.5, alpha=0.5, facecolor=rep_c, edgecolor = 'None') # R564

sns.lineplot(data=df, x='wav', y='value', style='treatment', style_order=['C', 'D'], color='black', linewidth=1.5)
plt.xlabel('Wavelength (nm)')
plt.ylabel('Reflectance')
labels = ['Water stress', 'Control']
handles, labels_2 = ax.get_legend_handles_labels()
green_patch = Patch(facecolor=pri_c, edgecolor='w',
                    label='Color Patch', alpha=0.9)
red_patch = Patch(facecolor=rep_c, edgecolor='w',
                    label='Color Patch', alpha=0.9)
handles, labels = ax.get_legend_handles_labels()
lgdn = ax.legend(title='', loc='center left', 
    fancybox=True, handles=[handles[0], handles[1], green_patch, red_patch],  
    labels=['Control','Water stress', 'PRI bands', 'REP bands'])

ax.yaxis.set_major_locator(ticker.MaxNLocator(4))
ax.xaxis.set_major_locator(ticker.MaxNLocator(4))

if save == True:
    output_fn = './graphs/Figure_1c'
    plt.savefig(output_fn + '.png', pad_inches=0, format='png', dpi=600, bbox_inches='tight')
    plt.savefig(output_fn+ '.pdf', pad_inches=0, format='pdf', dpi=600, bbox_inches='tight')

plt.show()