#%%
"""
Plot HSI PRI images with a 4x4 layout (Fig. 3 a)
"""

import os
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import tifffile
import matplotlib.patheffects as pe
from matplotlib.cm import ScalarMappable
import matplotlib.colors as clr
from data_directory_path import data_directory
from fig_colors import treatment_palette as c_palette 
import os
os.chdir(data_directory)

# Save plot? 
save = False

im_dir = './data/HSI/HSI_output/'

# get PRI and NDVI image tiff file names
pri_files = [f for f in os.listdir(im_dir + 'PRI-tiffs/') if f.endswith('pri_R564_im.tiff')]
ndvi_files = [f for f in os.listdir(im_dir + 'NDVI-tiffs/') if f.endswith('ndvi_im.tiff')]

# open PRI and NDVI tiff files and save arrays in a dictionary 
PRIs = {}
NDVIs = {}
for i in range(len(pri_files)):
    dt = pri_files[i][0:8]
    PRIs[dt] = tifffile.imread(im_dir + 'PRI-tiffs/' + pri_files[i])
    NDVIs[dt] = tifffile.imread(im_dir + 'NDVI-tiffs/' + ndvi_files[i])

#%%
# set font sizes
SMALL_SIZE = 12
MEDIUM_SIZE = 12
plt.rc('font', size=SMALL_SIZE) 
plt.rc('axes', titlesize=MEDIUM_SIZE)  
plt.rc('axes', labelsize=MEDIUM_SIZE) 
plt.rc('xtick', labelsize=SMALL_SIZE)   
plt.rc('ytick', labelsize=SMALL_SIZE)    
plt.rc('legend', fontsize=MEDIUM_SIZE)

# colormap based on min/max with quantiles colored
#Q1 = np.min(list(Q1s.values()))
#Q2 = np.median(list(Q2s.values()))
#Q3 = np.max(list(Q3s.values()))
#vmin = np.min(list(vmins.values()))
#vmax = np.max(list(vmaxs.values()))
Q1 = -0.06550757586956024
Q2 = -0.03080308623611927
Q3 = 0.02076452551409602
vmax =  0.12669869109988213
vmin = -0.18314990788698196

cmap = clr.LinearSegmentedColormap.from_list('custom_cmap', ['#ffffff', '#ededed', '#d1d1d1','#acacac',
                                                             '#828282','#5b5b5b','#2b2b2b','#000000',
                                                             '#9e0142','#9e0142','#d8434e','#f67b4a','#fdbf6f',
                                                             '#feeea2','#f1f9a9','#bfe5a0','#75c8a5',
                                                             '#388eba','#5e4fa2','#000000','#2b2b2b',
                                                             '#5b5b5b','#828282','#acacac','#d1d1d1',
                                                             '#ededed','#ffffff'])
#cmap = 'Spectral'

drought_days = ['20220815', '20220816', '20220819', '20220823', 
                '20220825', '20220829', '20220830', '20220831']

fig, axs = plt.subplots(4, 4, figsize=(15,15))
plt.subplots_adjust(hspace = 0.15, wspace= 0)

Q1s = {}
Q3s = {}
Q2s = {}
vmins = {}
vmaxs = {}

for pri, key, ax in zip(PRIs.values(), PRIs.keys(), axs.ravel()) :
    mask = np.ma.getmask(np.ma.masked_where(NDVIs[key]<0.25, NDVIs[key]))
    im = PRIs[key]
    im[mask == True] = np.nan
    Q1s[key] = np.nanpercentile(im, 25) 
    Q3s[key] = np.nanpercentile(im, 75) 
    Q2s[key] = np.nanpercentile(im, 50) 
    vmins[key] = np.nanpercentile(im, 1) 
    vmaxs[key] = np.nanpercentile(im, 99) 
    implot = ax.imshow(im, cmap=cmap, vmin=vmin, vmax=vmax)
    ax.set(xticks=[], yticks=[])
    if key in drought_days:
        ax.set_title(pd.to_datetime(key).strftime('%b %d'), fontdict={'color': c_palette[1], 'fontweight': 'bold'})
    else:
        ax.set_title(pd.to_datetime(key).strftime('%b %d'), fontdict={'color': c_palette[0], 'fontweight': 'bold'})
    ax.tick_params(bottom = False, left= False)

norm = plt.Normalize(vmin, vmax)
sm =  ScalarMappable(norm=norm, cmap=cmap)
sm.set_array([])
cax = axs[1,3].inset_axes([1.1, 0, 0.1, 1])
cbar = fig.colorbar(sm, cax=cax)
md=(Q1+Q3)/2
cbar.set_ticks([vmin,Q1, md, Q3,vmax])
cbar.set_ticklabels([round(vmin, 2), round(Q1, 2), round(Q2, 2),round(Q3,2),round(vmax, 2)])
cbar.ax.get_yaxis().labelpad = 15
cbar.ax.set_ylabel('PRI', rotation = 270)

# plant treatment labels
linew = 3
lab_c = 'black'
foreg_c = 'white'
font_dict = {'size':14, 'weight': 'bold'} 
axs[0,0].text(x=40, y= 130, s='D', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 54D
axs[0,0].text(x=420, y= 120, s='C', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 25C
axs[0,0].text(x=770, y= 120, s='D', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 10D
axs[0,0].text(x=770, y= 650, s='C', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 28C
axs[0,0].text(x=820, y= 990, s='D', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 9D
axs[0,0].text(x=470, y=920, s='C', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 6C
axs[0,0].text(x=120, y= 880, s='D', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 55D
axs[0,0].text(x=50, y= 610, s='C', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 30C

# set subfigures 3,2 and 3,3 empty
axs[3,2].axis('off')
axs[3,3].axis('off')

# insert label (a)
axs[0,0].text(-0.2, 0.99, '(a)', ha='left', va='top', transform=axs[0,0].transAxes,  
            fontdict ={'size':14, 'weight': 'bold'})


if save:
    output_fn = './graphs/Figure_3a__nosat'
    plt.savefig(output_fn + '.png', format='png', bbox_inches='tight')

plt.show()


# %%
