#%%
"""
Plot HSI REP images with a 4x4 layout (Fig. 4 a)
"""

import os
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import tifffile
import matplotlib.patheffects as pe
from matplotlib.cm import ScalarMappable
import matplotlib.colors as clr
from fig_colors import treatment_palette as c_palette 
from data_directory_path import data_directory
import os
os.chdir(data_directory)

# Save plot? 
save = True

im_dir = './data/HSI/HSI_output/'

# get REP and NDVI image tiff file names
ndvi_files = [f for f in os.listdir(im_dir + 'NDVI-tiffs/') if f.endswith('ndvi_im.tiff')]
rep_files = [f for f in os.listdir(im_dir + 'REP-tiffs/') if f.endswith('rep_im.tiff')]

# open REP and NDVI tiff files and save arrays in a dictionary 
NDVIs = {}
REPs = {}
for i in range(len(ndvi_files)):
    dt = ndvi_files[i][0:8]
    NDVIs[dt] = tifffile.imread(im_dir + 'NDVI-tiffs/' + ndvi_files[i])
    REPs[dt] = tifffile.imread(im_dir + 'REP-tiffs/' + rep_files[i])

#%%
# set font sizes
SMALL_SIZE = 12
MEDIUM_SIZE = 12
plt.rc('font', size=SMALL_SIZE) 
plt.rc('axes', titlesize=MEDIUM_SIZE)  
plt.rc('axes', labelsize=MEDIUM_SIZE) 
plt.rc('xtick', labelsize=SMALL_SIZE)   
plt.rc('ytick', labelsize=SMALL_SIZE)    
plt.rc('legend', fontsize=MEDIUM_SIZE)

# cmap colormap is based on min/max and quantiles colored
#Q1 = np.min(list(Q1s.values()))
#Q2 = np.median(list(Q2s.values()))
#Q3 = np.max(list(Q3s.values()))
#vmin = np.min(list(vmins.values()))
#vmax = np.max(list(vmaxs.values()))
Q1 = 719.896728515625
Q2 = 721.3208618164062
Q3 = 722.5176391601562
vmin = 712.0980059814453
vmax = 730.8817175292969


cmap = clr.LinearSegmentedColormap.from_list('custom_cmap', ['#ffffff','#ffffff', '#ededed', '#d1d1d1','#acacac',
                                                             '#828282','#5b5b5b','#2b2b2b','#000000',
                                                             '#9e0142','#d8434e','#f67b4a','#fdbf6f',
                                                             '#feeea2','#f1f9a9','#bfe5a0','#75c8a5',
                                                             '#388eba','#5e4fa2','#000000','#2b2b2b',
                                                             '#5b5b5b','#828282','#acacac','#d1d1d1',
                                                             '#ededed','#ffffff','#ffffff'])

drought_days = ['20220815', '20220816', '20220819', '20220823', 
                '20220825', '20220829', '20220830', '20220831']

fig, axs = plt.subplots(4, 4, figsize=(15,15))
plt.subplots_adjust(hspace = 0.15, wspace= 0)

Q1s = {}
Q3s = {}
Q2s = {}
vmins = {}
vmaxs = {}

for rep, key, ax in zip(REPs.values(), REPs.keys(), axs.ravel()) :
    mask = np.ma.getmask(np.ma.masked_where(NDVIs[key]<0.25, NDVIs[key]))
    im = REPs[key]
    im[mask == True] = np.nan
    Q1s[key] = np.nanpercentile(im, 25) 
    Q3s[key] = np.nanpercentile(im, 75) 
    Q2s[key] = np.nanpercentile(im, 50) 
    vmins[key] = np.nanpercentile(im, 1) 
    vmaxs[key] = np.nanpercentile(im, 99) 
    implot = ax.imshow(im, vmin=vmin, vmax=vmax, cmap=cmap)
    ax.set(xticks=[], yticks=[])
    if key in drought_days:
        ax.set_title(pd.to_datetime(key).strftime('%b %d'), fontdict={'color': c_palette[1], 'fontweight': 'bold'})
    else:
        ax.set_title(pd.to_datetime(key).strftime('%b %d'), fontdict={'color': c_palette[0], 'fontweight': 'bold'})
    ax.tick_params(bottom = False, left= False)
    #ax.text(x=50, y=990, s='Cont. = ' + str(np.round(cont, 2)), 
    #        path_effects=[pe.withStroke(linewidth=2, foreground='white')])

norm = plt.Normalize(vmin, vmax)
sm =  ScalarMappable(norm=norm, cmap=cmap)
sm.set_array([])
cax = axs[1,3].inset_axes([1.1, 0, 0.1, 1])
cbar = fig.colorbar(sm, cax=cax,)
md=(Q1+Q3)/2
cbar.set_ticks([vmin,Q1,Q2,Q3,vmax])
cbar.set_ticklabels(['%.1f '%vmin, '%.1f '%Q1, '%.1f '%md, '%.1f '%Q3,
                     '%.1f '%vmax])
cbar.ax.get_yaxis().labelpad = 15
cbar.ax.set_ylabel('REP (nm)', rotation = 270)

# plant treatment labels
linew = 3
lab_c = 'black'
foreg_c = 'white'
font_dict = {'size':14, 'weight': 'bold'} 
axs[0,0].text(x=40, y= 130, s='D', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 54D
axs[0,0].text(x=420, y= 120, s='C', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 25C
axs[0,0].text(x=770, y= 120, s='D', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 10D
axs[0,0].text(x=770, y= 650, s='C', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 28C
axs[0,0].text(x=820, y= 990, s='D', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 9D
axs[0,0].text(x=470, y=920, s='C', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 6C
axs[0,0].text(x=120, y= 880, s='D', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 55D
axs[0,0].text(x=50, y= 610, s='C', color=lab_c, fontdict = font_dict,
              path_effects=[pe.withStroke(linewidth=linew, foreground=foreg_c)]) # 30C

# set subfigures 3,2 and 3,3 empty
axs[3,2].axis('off')
axs[3,3].axis('off')

# insert label (a)
axs[0,0].text(-0.2, 0.99, '(a)', ha='left', va='top', transform=axs[0,0].transAxes,  
            fontdict ={'size':14, 'weight': 'bold'})


if save == True:
    output_fn = './graphs/Figure_4a__nosat'
    plt.savefig(output_fn + '.png', bbox_inches='tight', dpi=600)

plt.show()


# %%


# %%
