#%%
"""
Calculate Cohen’s d effect size Cohen’s d effect size between primary 
and secondary shoot (a) PRI and (b) REP values on three days (9 Aug, 31 Aug, 8 Sep) 
and plot barplot (Fig. 5) 
"""

import tifffile
import numpy as np
import scipy as scipy
from math import sqrt
import pandas as pd
import math as math
import seaborn as sns
import matplotlib.pyplot as plt
from fig_colors import treatment_palette as c_palette
from data_directory_path import data_directory
import os
os.chdir(data_directory)

# Save plot? 
save = True

def cohend(d1, d2):
    """
    function to calculate Cohen's d for independent samples
    """
    n1, n2 = len(d1), len(d2)
    # variance 
    s1, s2 = np.nanvar(d1, ddof=1), np.nanvar(d2, ddof=1)
    
    # check that variance ratio is not < 1:¤ 
    print('variance ratio:' + str(round(s1/s2, 3)))
    
    # pooled standard deviation
    s = math.sqrt(((n1 - 1) * s1 + (n2 - 1) * s2) / (n1 + n2 - 2))
    # means of the samples
    u1, u2 = np.nanmean(d1), np.nanmean(d2)

    # return effect size
    return abs((u1 - u2) / s)

#%%
### read pri, rep, and mask tiffs
pri_files = ['20220809-pri_R564_im.tiff', '20220831-pri_R564_im.tiff', '20220908-pri_R564_im.tiff']
rep_files = ['20220809-rep_im.tiff', '20220831-rep_im.tiff', '20220908-rep_im.tiff']
mask_files = ['20220809-mask.tiff', '20220831-mask.tiff', '20220908-mask.tiff']

pri_paths = ['./data/HSI/HSI_output/PRI-tiffs/' + f for f in pri_files]
rep_paths = ['./data/HSI/HSI_output/REP-tiffs/' + f for f in rep_files]
mask_paths = ['./data/HSI/HSI_output/masks/' + f for f in mask_files]

pri_ims = {}
for f in pri_paths:
    dt = ''.join(f.split('-tiffs/')[1].split('-')[0])
    temp = tifffile.imread(f)
    pri_ims[dt] = temp 

rep_ims ={}
for f in rep_paths:
    dt = dt = ''.join(f.split('-tiffs/')[1].split('-')[0])
    temp = tifffile.imread(f)
    rep_ims[dt] = temp 

masks = {}
for f in mask_paths:
    dt = ''.join(f.split('masks/')[1].split('-')[0])
    temp = tifffile.imread(f)
    masks[dt] = temp 
    
plant_ids = [54,25,10,30,28,9,6,55]
shoot_dict = {601:'', 602:'', 2501:'',2502:'', 2801:'',2802:'', 3001:'',3002:'', 901:'',902:'', 1001:'',1002:'', 5401:'',5402:'', 5501:'', 5502:''}

### calculate pri and rep stats 
pri_stats = {}
rep_stats = {}
pri_data = {}
rep_data = {}

for key in pri_ims.keys():
    im_pri = pri_ims[key]
    im_rep = rep_ims[key]
    shoot_mask = masks[key]

    stats_temp_pri = []
    for plant in plant_ids:
        d1 = im_pri[shoot_mask==int(str(plant) + '01')]
        d1 = d1[~np.isnan(d1)]
        d2 = im_pri[shoot_mask==int(str(plant) + '02')]
        d2 = d2[~np.isnan(d2)]

        d = cohend(d1, d2)
        s, p = scipy.stats.ttest_ind(d1, d2, alternative="less")

        new_row = {'plant':plant, 'cohen_d':d, 'one-tailed t':s, 'p-value':p}
        stats_temp_pri.append(new_row)

    stats_temp_rep = []
    for plant in plant_ids:
        d1 = im_rep[shoot_mask==int(str(plant) + '01')]
        d1=d1[~np.isnan(d1)]
        d2 = im_rep[shoot_mask==int(str(plant) + '02')]
        d2=d2[~np.isnan(d2)]

        d = cohend(d1, d2)
        s, p = scipy.stats.ttest_ind(d1, d2, alternative="less")

        new_row = {'plant':plant, 'cohen_d':d, 'one-tailed t':s, 'p-value':p}
        stats_temp_rep.append(new_row)

    pri_stats[key] = pd.DataFrame.from_dict(stats_temp_pri)
    rep_stats[key] = pd.DataFrame.from_dict(stats_temp_rep)
    
    pri_data[key] = {601:'', 602:'', 2501:'',2502:'', 2801:'',2802:'', 3001:'',3002:'', 901:'',902:'', 1001:'',1002:'', 5401:'',5402:'', 5501:'', 5502:''}
    rep_data[key] = {601:'', 602:'', 2501:'',2502:'', 2801:'',2802:'', 3001:'',3002:'', 901:'',902:'', 1001:'',1002:'', 5401:'',5402:'', 5501:'', 5502:''}

    for shoot_key in pri_data[key].keys():
        im_pri = pri_ims[key]
        temp_pri = im_pri[shoot_mask==int(shoot_key)]
        pri_data[key][shoot_key] = temp_pri.data

    for shoot_key in rep_data[key].keys():
        im_rep = rep_ims[key]
        temp_rep = im_rep[shoot_mask==int(shoot_key)]
        rep_data[key][shoot_key] = temp_rep.data

keys = list(pri_stats.keys())
f = lambda x: list(x) if len(x) > 1 else x
df_pri = (pd.concat([pri_stats[key].assign(Date=key) for key in keys]).groupby(['plant', 'cohen_d', 'one-tailed t', 'p-value'])['Date'].agg(f).reset_index())
df_rep = (pd.concat([rep_stats[key].assign(Date=key) for key in keys]).groupby(['plant', 'cohen_d', 'one-tailed t', 'p-value'])['Date'].agg(f).reset_index())

def treatment_lab(plant):
    if (plant == 6) | (plant == 30) | (plant == 25) | (plant == 28):
        return 'C'
    else:
        return 'D'
    
df_pri['Treatment'] = df_pri['plant'].apply(treatment_lab)
df_pri['Index'] = 'PRI'
df_rep['Treatment'] = df_rep['plant'].apply(treatment_lab)
df_rep['Index'] = 'REP'

df = pd.concat([df_pri, df_rep], axis=0)

edge_c = 'black'
err_c = 'black'
c_palette = ['#b4e2ba', '#fece65', '#e1640e']

# font sizes
SMALL_SIZE = 12
MEDIUM_SIZE = 14
BIGGER_SIZE = 16
plt.rc('font', size=MEDIUM_SIZE) 
plt.rc('axes', titlesize=MEDIUM_SIZE)  
plt.rc('axes', labelsize=MEDIUM_SIZE) 
plt.rc('xtick', labelsize=SMALL_SIZE)   
plt.rc('ytick', labelsize=SMALL_SIZE)    
plt.rc('legend', fontsize=SMALL_SIZE)    

shoot_ids = ['6C', '25C', '28C', '30C', '9D', '10D', '54D', '55D']

ylabs = ["PRI shoot location\neffect size\n(Cohen's d)", "REP shoot location\neffect size\n(Cohen's d)"]
titles = ['(a)', '(b)']
fig, axs = plt.subplots(2, 1, figsize=(9,5)) 

for index, ylab, title, ax in zip(df['Index'].unique(), ylabs, titles, axs.ravel()):
    bxplt = sns.barplot(ax=ax, data=df[df['Index']==index], x='plant', y='cohen_d', hue='Date',
                        linewidth=0.5,
                        edgecolor=edge_c, errcolor=err_c, order=[6, 25, 28, 30, 9, 10, 54, 55],
                        palette=c_palette, alpha=0.9,
                        errorbar=("ci", 95))


    for patch in bxplt.patches:
        r, g, b, a = patch.get_facecolor()
        patch.set_facecolor((r, g, b, .5))

    ax.set_ylim(0,1)
    ax.set(xlabel='', ylabel=ylab)
    ax.set_xticklabels(shoot_ids)
    ax.set_yticks(ticks=[0, 0.5, 1])

    ax.text(0.02, .95, title, ha='left', va='top', transform=ax.transAxes, 
            fontdict ={'size':12, 'weight': 'bold'})

    lgnd = ax.legend()
    lgnd.remove()

# plat 55D  REP on sep 08 is not normally distributed, not this with '*'
axs[1].text(0.965, .005, '*', transform=ax.transAxes, 
    fontdict ={'size':12})

handles, labels = axs[0].get_legend_handles_labels()
axs[0].legend(handles=handles, 
              labels=['Pre-WR,\nAug 09', 'Height of WR,\nAug 31', 'Recovery,\nSep 08'], 
              ncol=3)
axs[1].set(xlabel='Plant ID')

plt.tight_layout()

if save:
    output_fn = './graphs/Figure_5__nosat'
    plt.savefig(output_fn + '.png', bbox_inches='tight', dpi=600)
    plt.savefig(output_fn + '.pdf', bbox_inches='tight', dpi=600)

plt.show()

# %%
