#%%
"""
Calculate Pearson's r between optical indices PRI and REP and other measurements 
and plot barplot (Fig. 6)
"""

import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd
from scipy.stats import pearsonr
from data_directory_path import data_directory
import os
os.chdir(data_directory)

# Save plot?
save = False

def pearsonr_pval(x,y):
        return pearsonr(x,y)[1]

def p_lims(pval):
    if pval <= 0.01 : 
        return '***'
    elif (pval <= 0.05) & (pval >= 0.01):
        return '**'
    elif (pval <= 0.1) & (pval >= 0.05):
        return '*'
    else: 
        return '.'

#%%

data = pd.read_csv('./data/data_all__nosat.csv')
data['AB_carot'] = data['a-CAROT'] + data['B-CAROT']

# Set font sizes
SMALL_SIZE = 12
MEDIUM_SIZE = 14
BIGGER_SIZE = 16
plt.rc('font', size=MEDIUM_SIZE) 
plt.rc('axes', titlesize=MEDIUM_SIZE)  
plt.rc('axes', labelsize=MEDIUM_SIZE) 
plt.rc('xtick', labelsize=SMALL_SIZE)   
plt.rc('ytick', labelsize=SMALL_SIZE)    
plt.rc('legend', fontsize=SMALL_SIZE)    

c_palette = ['#b4e2ba', '#fece65']

params_1 = ['CONC VIO', 'CONC ANTERA', 'CONC ZEAX', 'CONC LUT', 'AB_carot',
             'TOTAL CAROT', 'A+Z/VAZ', 'CLORF a+b', 'CAROT TOT/CLORF (a+b)'] 
axis_1 = ['V\n(µg cm$\mathregular{^{-2}}$)',
          'A\n(µg cm$\mathregular{^{-2}}$)',
          'Z\n(µg cm$\mathregular{^{-2}}$)', 
          'L\n(µg cm$\mathregular{^{-2}}$)', 
          'Carotenes\n(µg cm$\mathregular{^{-2}}$)',
          'Total carot.\n(µg cm$\mathregular{^{-2}}$)',
          'DEPS\n(A+Z/VAZ)',
          'Chl a+b\n(µg cm$\mathregular{^{-2}}$)',
          'Total carot.\n/ Chl a+b']

params_2 = ['LWP', 'SM', 'YII_MAX', 'qLs', 'qLr', 'qLT',
            'Y_NPQs','Y_NPQr','Y_NPQ','E_correct', 'A_correct']
axis_2 = ['$\mathregular{Ψ{_W}}$',
          'SM-%',
          'Φ$\mathit{PSII_{MAX}}$',
          'qL$\mathregular{_{s}}$',
          'qL$\mathregular{_{r}}$',
          'qL$\mathregular{_{T}}$',
          'Φ$\mathit{NPQ_{S}}$',
          'Φ$\mathit{NPQ_{R}}$',
          'Φ$\mathit{NPQ}$',
          '$\mathit{E}$\n(μmol m$\mathregular{^{-2}})$',
          '$\mathit{A}$\n(μmol m$\mathregular{^{-2}}$)']

params_list = [params_1, params_2]

titles = ['(a)', '(b)']

corr_mat = data.corr(method='pearson', numeric_only=True)
pvalues = data.corr(method=pearsonr_pval, numeric_only=True)

corr_indices = corr_mat[['PRI_circle', 'REP_circle']]
corr_indices = corr_indices.rename_axis('measurement').reset_index()
corr_indices = pd.melt(corr_indices, id_vars='measurement', value_name='rp')
pval_indices = pvalues[['PRI_circle', 'REP_circle']]
pval_indices = pval_indices.rename_axis('measurement').reset_index()
pval_indices = pd.melt(pval_indices, id_vars='measurement', value_name='pval')
corr_indices =pd.merge(corr_indices, pval_indices, on=['measurement', 'variable'])
corr_indices['pval_lims'] = corr_indices['pval'].apply(p_lims)

fig, axs = plt.subplots(2,1, figsize=(12,5))

for params, title, ax in zip(params_list, titles,  axs.ravel()):
    df_temp = corr_indices[corr_indices['measurement'].isin(params)].reset_index()
    bxplt= sns.barplot(data = df_temp, 
                       x='measurement', y='rp', hue='variable', edgecolor='black',
                       ax=ax, linewidth=0.5, alpha=0.9,
                       order = params, palette=c_palette)
    

    for patch in bxplt.patches:
        r, g, b, a = patch.get_facecolor()
        patch.set_facecolor((r, g, b, .5))
    
    ax.set_ylim(-1.1, 1.1)
    ax.set_ylabel("Pearson's $\mathit{r}$")

    ax.text(x=0.96, y=0.85, s=title , transform = ax.transAxes,  
            fontdict ={'size':12, 'weight': 'bold'})

    lgnd = ax.legend()
    lgnd.remove()

    for container in ax.containers:
        ax.bar_label(container, fmt='%.2f', padding=2, fontsize=8)

    ax.set_xlabel('')

    ax.set_yticks([-1, 0, 1])

axs[0].set_xticklabels(axis_1)
axs[1].set_xticklabels(axis_2)

handles, labels = axs[1].get_legend_handles_labels()
axs[1].legend(handles=handles, labels=['PRI', 'REP'])

plt.tight_layout()   
if save == True:
    output_fn = './graphs/Figure_6__nosat'
    plt.savefig(output_fn + '.png',  bbox_inches='tight', dpi=600)
    plt.savefig(output_fn + '.pdf',  bbox_inches='tight', dpi=600)

plt.show()


# %%
