#%%
"""
Plot chlorophyll fluoresence parameters (Appendix Fig. A2)
Timeseries F0, F_M, YNPQ_S, qLs and barplot YPSII, YNPQ_R 
"""

import matplotlib.pyplot as plt
import pandas as pd
from matplotlib import ticker
import numpy as np
import matplotlib.dates as mdates
from scipy.stats import ranksums
import seaborn as sns
from fig_colors import treatment_palette as c_palette 
from data_directory_path import data_directory
import os
os.chdir(data_directory)

# save plot?
save = True

# remove outlier based on Fm?
rm_outliers = False

def p_lims(pval):
    if pval <= 0.01 : 
        return '***'
    elif (pval <= 0.05) & (pval >= 0.01):
        return '**'
    elif (pval <= 0.1) & (pval >= 0.05):
        return '*'
    else: 
        return 'ns'
    
data = pd.read_csv('./data/data_all__nosat.csv', sep=',')
data_bar = data[(data['Date'] == 20220809) |(data['Date'] == 20220831) | (data['Date'] == 20220908)]
data['Pot']  =  round(data['Pot']).astype(str)
data['Date'] = pd.to_datetime(data['Date'], format='%Y%m%d')

#%%
if rm_outliers:
    # remove outliers based on Fm
    Q1 = np.percentile(data['Fm.'], 25)
    Q3 = np.percentile(data['Fm.'], 75)
    IQR = Q3 - Q1
    min_val = Q1 - 1.5*IQR
    max_val = Q3 + 1.5*IQR

    removed_data = data[(data['Fm.'] < min_val) | (data['Fm.'] > max_val)] 
    data = data[(data['Fm.'] >= min_val) & (data['Fm.'] <= max_val)] 

cols = ['F0', 'Fm.', 'YII_MAX',
        "F's", "F'm",  'YII',
        'qLs', 'qLr', 'qLT',
        'Y_NPQs', 'Y_NPQr', 'Y_NPQ',
        'NPQs',   'NPQr', 'NPQ'] 
ts_cols = ['F0', 'Fm.', 'YII_MAX',
           'qLs','Y_NPQs','NPQs'] 

titles = ['(a)', '(b)', '(c)', '(d)', '(e)', '(f)', '(g)', '(h)', '(i)', '(j)', '(k)', '(l)', '(m)', '(n)', '(o)', '(p)', '(q)']
y_labs = ['$\mathit{F_{0}}$', '$\mathit{F_{M}}$', 'Φ$\mathit{PSII_{MAX}}$',
          "$\mathit{F'_{S}}$", "$\mathit{F'_{M}}$", 'Φ$\mathit{PSII}$',
          '$\mathit{qLs}$', '$\mathit{qLr}$', '$\mathit{qLT}$',
          'Φ$\mathit{NPQ_{S}}$', 'Φ$\mathit{NPQ_{R}}$', 'Φ$\mathit{NPQ}$',
          '$\mathit{K_{NPQs}}$','$\mathit{K_{NPQr}}$','$\mathit{K_{NPQ}}$']

# set font sizes
SMALL_SIZE = 12
MEDIUM_SIZE = 14
BIGGER_SIZE = 16
plt.rc('font', size=MEDIUM_SIZE) 
plt.rc('axes', titlesize=MEDIUM_SIZE)  
plt.rc('axes', labelsize=MEDIUM_SIZE) 
plt.rc('xtick', labelsize=SMALL_SIZE)   
plt.rc('ytick', labelsize=SMALL_SIZE)    
plt.rc('legend', fontsize=SMALL_SIZE)    

fig, axs = plt.subplots(nrows=5, ncols=3, figsize=(13, 14))
plt.subplots_adjust(hspace=0.4, wspace= 0.2)

for col, ax, ylab, title in zip(cols, axs.ravel(), y_labs, titles) :
    if col in ts_cols:
        sns.lineplot(data, x='Date', y=col, 
                    hue='Treatment', hue_order=['C', 'D'], palette=c_palette, 
                    errorbar=('ci', 95), ax=ax)

        ax.text(0.02, .96, title, ha='left', va='top', transform=ax.transAxes, 
        fontdict ={'size':14, 'weight': 'bold'}) 
        ax.set_xlabel('')
        ax.set_ylabel(ylab)
        handles, labels = ax.get_legend_handles_labels()
        if col == cols[2]:
            handles, labels = ax.get_legend_handles_labels()
            lgdn = ax.legend(title='', loc='center left', 
                            fancybox=True, 
                            handles=[handles[0], handles[1]],  
                            labels=['Control','Water stress'],
                            bbox_to_anchor=(0, 0.4))
        else:
            lgdn = ax.legend()
            lgdn.remove()
        y_min, y_max = ax.get_ylim() 
        diff = y_max - y_min
        ax.yaxis.set_major_locator(ticker.MaxNLocator(4))
        ax.scatter(pd.to_datetime('2022-08-14 12:00'), 
                y_min+(diff*0.08), c=c_palette[1],  linewidth=0.2,
                marker=r'$\downarrow$',s=300, label='Irrigation\nresumed', zorder=2)
        ax.scatter(pd.to_datetime('2022-08-30 12:00'), 
                y_min+(diff*0.08), c=c_palette[0], linewidth=0.2,
                marker=r'$\downarrow$',s=300, label='Irrigation\nresumed', zorder=2)
        ax.set_xticks(ticks=[pd.to_datetime('2022-08-15'),
                            pd.to_datetime('2022-08-31')])
        ax.xaxis.set_major_formatter(mdates.DateFormatter('%b %d'))

    else:
        edge_c = 'black'
        sns.barplot(ax=ax, data = data_bar, x='Date', y=col, 
                    hue='Treatment', hue_order= ['C', 'D'],palette=c_palette, 
                    linewidth=1, edgecolor=None, errcolor=edge_c, alpha=0.35,
                    errorbar=("ci", 95))
        ax = sns.stripplot(ax=ax, data = data_bar, x='Date', y=col, 
                        edgecolor=None, linewidth=0.8, palette=c_palette, 
                        hue='Treatment',
                        dodge=True, size=6)
        ax.set(xlabel='', ylabel=ylab)
        ax.text(0.02, .96, title, ha='left', va='top', transform=ax.transAxes, 
                fontdict ={'size':14, 'weight': 'bold'}) 
        
        max_val = data_bar[col].max() 
        ax.set_ylim(0, max_val*1.3)
        dates = data_bar['Date'].unique()
        x1_locs = [-0.25, 0.75, 1.75]
        x2_locs = [0.25, 1.25, 2.25]
        y, h = (max_val + max_val *0.05), (max_val*0.05)
        
        for dt, x1, x2 in zip(dates, x1_locs, x2_locs):
            wilcoxon_p = ranksums(data_bar[(data_bar['Treatment'] == 'D') & (data_bar['Date'] == dt)][col], 
                                    data_bar[(data_bar['Treatment'] == 'C')  & (data_bar['Date'] == dt) ][col])[1]
            wilcoxon_p = p_lims(wilcoxon_p)
            ax.plot([x1, x1, x2, x2], [y, y+h, y+h, y], lw=1.5, color='black')
            ax.text((x1+x2)*.5, y+h, wilcoxon_p, ha='center', va='bottom')

        ax.yaxis.set_major_locator(ticker.MaxNLocator(3))
        xticks = ['Aug 09', 'Aug 31', 'Sep 08']
        ax.set_xticklabels(xticks)

        lgnd = ax.legend()
        lgnd.remove()

plt.tight_layout()

if save:
    output_fn = './graphs/Figure_A2__nosat'
    plt.savefig(output_fn + '.png',  bbox_inches='tight', dpi=600)
    plt.savefig(output_fn + '.pdf',  bbox_inches='tight', dpi=600)
    print('figure saved')
    
plt.show()

# %%
