"""
Plot GFS gas echnage (Appendix Fig. A3) with wilcoxon rank sum p 
"""
#%%
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
from matplotlib import ticker
from fig_colors import treatment_palette as c_palette
from scipy.stats import ranksums
from data_directory_path import data_directory
import os
os.chdir(data_directory)

# Save plot?
save = True

def p_lims(pval):
    if pval <= 0.01 : 
        return '***'
    elif (pval <= 0.05) & (pval >= 0.01):
        return '**'
    elif (pval <= 0.1) & (pval >= 0.05):
        return '*'
    else: 
        return 'ns'

#%%

data = pd.read_csv('./data/data_all.csv', sep=',')
data = data[(data['Date'] == 20220809) |(data['Date'] == 20220831) | (data['Date'] == 20220908)]
data['Date'] = data['Date'].astype(str)

# set font sizes
SMALL_SIZE = 12
MEDIUM_SIZE = 14
BIGGER_SIZE = 16
plt.rc('font', size=MEDIUM_SIZE) 
plt.rc('axes', titlesize=MEDIUM_SIZE)  
plt.rc('axes', labelsize=MEDIUM_SIZE) 
plt.rc('xtick', labelsize=SMALL_SIZE)   
plt.rc('ytick', labelsize=SMALL_SIZE)    
plt.rc('legend', fontsize=SMALL_SIZE)   

cols = ['A_correct', 'E_correct', 'GH2O_correct', 'VPD']
xtick = ['Aug 09', 'Aug 31', 'Sep 08']
ylabs = ['$\mathit{A}$ (μmol m$\mathregular{^{-2}}$ s$\mathregular{^{-1}}$)',
         '$\mathit{E}$ (mmol m$\mathregular{^{-2}}$ s$\mathregular{^{-1}}$)',
         'GH$\mathregular{_{2}}$O\n(mmol m$\mathregular{^{-2}}$ s$\mathregular{^{-1}}$)',
         'VPD (Pa/kPa)']
titles = ['(a)','(b)','(c)','(d)']

fig, axs = plt.subplots(1, 2, figsize=(9,3)) 

edge_c = None
err_c = 'black'

for col, ylab, title, ax in zip(cols, ylabs, titles, axs.ravel()):
    sns.barplot(ax=ax, data = data, x='Date', y=col, 
                hue='Treatment', hue_order= ['C', 'D'], palette=c_palette, 
                edgecolor=edge_c, errcolor=err_c, alpha=0.35,errwidth=1.5,
                errorbar=("ci", 95))
    ax = sns.stripplot(ax=ax, data = data, x='Date', y=col, 
                       edgecolor=edge_c, linewidth=0.8,
                       hue='Treatment', palette=c_palette,
                       dodge=True, size=6)
    ax.set(xlabel= '', ylabel=ylab)
    ax.text(x=-0.11, y=0.9, s=title , transform = ax.transAxes,  
            fontdict ={'size':12, 'weight': 'bold'})
    ax.set_xticklabels(xtick)
    lgnd = ax.legend()
    lgnd.remove()

    ax.yaxis.set_major_locator(ticker.MaxNLocator(3))

    max_val, min_val = data[col].max(), data[col].min() 
    ax.set_ylim(min_val*1.3, max_val*1.3)
    dates = data['Date'].unique()
    x_ints = list(range(0, len(dates)))
    x1_locs = [i-0.25 for i in x_ints]
    x2_locs = [i+0.25 for i in x_ints]
    y, h = (max_val + max_val *0.05), (max_val*0.05)
    for dt, x1, x2 in zip(dates, x1_locs, x2_locs):
        wilcoxon_p = ranksums(data[(data['Treatment'] == 'D') & (data['Date'] == dt)][col], 
                            data[(data['Treatment'] == 'C')  & (data['Date'] == dt) ][col])[1]
        wilcoxon_p = p_lims(wilcoxon_p)

        ax.plot([x1, x1, x2, x2], [y, y+h, y+h, y], lw=1.5, color='black')
        ax.text((x1+x2)*.5, y+h, wilcoxon_p, ha='center', va='bottom')

handles, labels = axs[0].get_legend_handles_labels()

axs[1].legend(loc='center right', 
                labels= ['Control', 'Water stress'],
                handles = handles, 
                ncol = 1,
                title='',
                borderpad=0.3,
                bbox_to_anchor=(1, 0.45))

plt.tight_layout()

if save == True:
    output_fn = './graphs/Figure_A3'
    plt.savefig(output_fn + '.png',  bbox_inches='tight', dpi=600)
    plt.savefig(output_fn + '.pdf',  bbox_inches='tight', dpi=600)

plt.show()