#%%
"""
Plot pigment concentrations (Appendix Fig. A4) with wilcoxon rank sum p 
"""
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
from fig_colors import treatment_palette as c_palette
from scipy.stats import ranksums
from matplotlib import ticker
from data_directory_path import data_directory
import os
os.chdir(data_directory)

# Save plot?
save = True

def p_lims(pval):
    if pval <= 0.01 : 
        return '***'
    elif (pval <= 0.05) & (pval >= 0.01):
        return '**'
    elif (pval <= 0.1) & (pval >= 0.05):
        return '*'
    else: 
        return 'ns'

#%%

data = pd.read_csv('./data/data_all.csv', sep=',')
data = data[(data['Date'] == 20220831) | (data['Date'] == 20220908)]
data['Date'] = data['Date'].astype(str)

# set font sizes
SMALL_SIZE = 12
MEDIUM_SIZE = 14
BIGGER_SIZE = 16
plt.rc('font', size=MEDIUM_SIZE) 
plt.rc('axes', titlesize=MEDIUM_SIZE)  
plt.rc('axes', labelsize=MEDIUM_SIZE) 
plt.rc('xtick', labelsize=SMALL_SIZE)   
plt.rc('ytick', labelsize=SMALL_SIZE)    
plt.rc('legend', fontsize=SMALL_SIZE)   

cols = ['CONC ANTERA', 'CONC ZEAX', 'CONC VIO', 
        'A+Z/VAZ', 'DEPS (A0.5+Z/VAZ)',  'a-CAROT',
        'B-CAROT','CONC LUT', 'CONC LX',
        'TOTAL CAROT','CLORF a+b', 'CAROT TOT/CLORF (a+b)',
        ]

xtick = ['Height of WS,\nAug 31', 'Recovery,\nSep 08']

ylabs = ['Antheraxanthin\n(µg cm$\mathregular{^{-2}}$)',
         'Zeaxanthin\n(µg cm$\mathregular{^{-2}}$)',
         'Violaxanthin\n(µg cm$\mathregular{^{-2}}$)',
         'A+Z / VAZ',
         'DEPS\n(A*0.5 + Z / VAZ)',
         'α-carotene\n(µg cm$\mathregular{^{-2}}$)',
         'β-carotene\n(µg cm$\mathregular{^{-2}}$)',
         'Lutein\n(µg cm$\mathregular{^{-2}}$)',
         'Lutein epoxide\n(µg cm$\mathregular{^{-2}}$)',
         'Total Carotenoids\n(µg cm$\mathregular{^{-2}}$)',
         'Total Chlorophyll\n(µg cm$\mathregular{^{-2}}$)',
         'Carotenoids/\nChlorophyll',
]

ylim_max = [0.6, 0.6, 2, 1, 1, 1.1, 2, 4.5, 0.09, 11, 55, 0.35]

titles = ['(a)','(b)','(c)','(d)','(e)',
          '(f)','(g)','(h)','(i)','(j)',
          '(k)', '(l)', '(m)', '(n)']

fig, axs = plt.subplots(4, 3, figsize=(10.5,11)) 

plt.subplots_adjust(hspace = 0.3, wspace= 0.25)

edge_c = None
err_c = 'black'

for col, ylab, ylim, title, ax in zip(cols, ylabs, ylim_max, titles, axs.ravel()):
    sns.barplot(ax=ax, data = data, x='Date', y=col, 
                hue='Treatment', hue_order= ['C', 'D'], palette=c_palette, 
                edgecolor=edge_c, errcolor=err_c, alpha=0.35,errwidth=1.5,
                errorbar=("ci", 95))
    ax = sns.stripplot(ax=ax, data = data, x='Date', y=col, 
                       edgecolor=edge_c, linewidth=0.8,
                       hue='Treatment', palette=c_palette, 
                       dodge=True, size=6)
    ax.set(xlabel='', ylabel=ylab)
    ax.text(x=0.01, y=.9, s=title , 
            transform = ax.transAxes,  
            fontdict ={'size':12, 'weight': 'bold'})
    ax.set_ylim(0, ylim)
    ax.set_xticklabels(xtick)
    lgnd = ax.legend()
    lgnd.remove()
    
    max_val = data[col].max() 
    #ax.set_ylim(0, max_val*1.8)
    dates = data['Date'].unique()
    x_ints = list(range(0, len(dates)))
    x1_locs = [i-0.25 for i in x_ints]
    x2_locs = [i+0.25 for i in x_ints]
    y, h = (ylim - ylim *0.2), (ylim*0.05)
    for dt, x1, x2 in zip(dates, x1_locs, x2_locs):
        wilcoxon_p = ranksums(data[(data['Treatment'] == 'D') & (data['Date'] == dt)][col], 
                            data[(data['Treatment'] == 'C')  & (data['Date'] == dt) ][col])[1]
        wilcoxon_p = p_lims(wilcoxon_p)

        ax.plot([x1, x1, x2, x2], [y, y+h, y+h, y], lw=1.5, color='black')
        ax.text((x1+x2)*.5, y+h, wilcoxon_p, ha='center', va='bottom')

    ax.yaxis.set_major_locator(ticker.MaxNLocator(3))


handles, labels = axs[3,1].get_legend_handles_labels()

axs[2,0].legend(loc='lower left', 
                labels= ['Control', 'WS'],
                handles = handles, 
                ncol = 1,
                title='',
                borderpad=0.3)


fig.tight_layout()

if save == True:
    output_fn = './graphs/Figure_A4'
    plt.savefig(output_fn + '.png',  bbox_inches='tight', dpi=600)
    plt.savefig(output_fn + '.pdf',  bbox_inches='tight', dpi=600)

plt.show()