#%%
"""
Calculate average spectra over a circle mask (and save as csv)
Plot average spectras by treatment (Appendix Fig. 9)
"""
import hsi_functions as hsi
import hsi_shoot_locs as slocs
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
import glob
from matplotlib.lines import Line2D
import numpy as np
from fig_colors import treatment_palette as c_palette
from data_directory_path import data_directory
import os
os.chdir(data_directory)

# save avergae spectras as csv?
save_spec = False

# save plot?
save_plot = True

def circle_av(im, center, radius):
    """
    Function that find and averages a cube over a cirlce mask defined by radious and center 
    """
    h = im.shape[1]
    w = im.shape[2]
    Y, X = np.ogrid[:h, :w]
    dist_from_center = np.sqrt((X - center[0])**2 + (Y-center[1])**2)

    mask = dist_from_center >= radius

    num_bands = im.shape[0]
    avg_values = np.zeros(num_bands)

    for band in range(num_bands):
        band_slice = im[band]
        mask_arr = np.ma.masked_array(band_slice, mask=mask)
        avg_values[band] = np.nanmean(mask_arr)

    return avg_values

dates = ['20220809', '20220815', '20220816', '20220819', '20220823', 
         '20220825', '20220829', '20220830', '20220831', '20220902',
         '20220903', '20220905', '20220907', '20220908']

#%%

image_files = glob.glob('./data/HSI/HSI_output/refl_HSI_data/*.dat')
image_files = [fn.replace('.dat', '') for fn in image_files]

shoot_locs = slocs.sloc_dict

pots = [54, 25, 10, 28, 9, 6, 55, 30]

all_spectra = pd.DataFrame({'wav': hsi.get_wav(image_files[0])[0]})

for im_file, date in zip(image_files, dates):
    print('reading ' + date)
    wav, fwhm = hsi.get_wav(im_file)
    spectra = pd.DataFrame({'wav': wav})

    im = hsi.read_img(im_file)
    locs = shoot_locs[date]

    for i, pot in zip(locs, pots):
        if pot in [54, 10, 9, 55]:
            spectra[date + '_' + str(pot) + '__D'] = circle_av(im, i, 50)
        else:
            spectra[date + '_' + str(pot) + '__C'] = circle_av(im, i, 50)
    
    all_spectra = pd.merge(all_spectra, spectra, how='outer')
    print('done with ' + date)

if save_spec == True:
    output_fn = './data/HSI/HSI_output/av_spectra.csv'
    all_spectra.to_csv(output_fn)
    print('csv saved')


#%%
spec_df = pd.read_csv('./data/HSI/HSI_output/av_spectra.csv')

spec_df.loc[(spec_df['wav'] > 600) & (spec_df['wav'] < 625)] = np.nan
spec_df.loc[(spec_df['wav'] >= 635.7) & (spec_df['wav'] <= 	657.1)] = np.nan

df_C_long = pd.melt(spec_df,id_vars= 'wav', 
                    value_vars = [t for t in spec_df.columns if all(char in t for char in '_C')])

df_D_long = pd.melt(spec_df,id_vars= 'wav', 
                    value_vars = [t for t in spec_df.columns if all(char in t for char in '_D')])

fig, axs = plt.subplots(4,4, figsize=(10,10))

for date, ax, in zip(dates, axs.ravel()):
    df_C_temp = df_C_long[df_C_long['variable'].str.contains(date)]
    df_D_temp = df_D_long[df_D_long['variable'].str.contains(date)]
    sns.lineplot(ax=ax, data=df_C_temp, x='wav', y='value',
                 errorbar=('ci', 95), color=c_palette[0])
    sns.lineplot(ax=ax, data=df_D_temp, x='wav', y='value',
                 errorbar=('ci', 95), color=c_palette[1])
    ax.set_ylim(0,1)
    ax.set(title=pd.to_datetime(date, format='%Y%m%d').strftime('%b %d'))
    ax.set_xlabel('')
    ax.set_ylabel('')
    ax.set_yticks(ticks=[0, 0.5, 1])
    ax.set_xticks(ticks=[500, 650, 800])


handles = [Line2D([0], [0], color=c_palette[0]),
           Line2D([0], [0], color=c_palette[1])]
labels = ['Control', 'Water stress']

lgdn = axs[0, 0].legend(title='', loc='upper left', 
                fancybox=True, handles = handles, labels = labels)
    
axs[3,2].axis('off')
axs[3,3].axis('off')

plt.tight_layout()

if save_plot == True:
    output_fn = './graphs/Figure_A9'
    plt.savefig(output_fn + '.png',  bbox_inches='tight', dpi=600)
    plt.savefig(output_fn + '.pdf',  bbox_inches='tight', dpi=600)

plt.show()


