"""
Module for processing summer 2022 HSI images
"""

import os
import numpy as np
import math as math
import matplotlib.pyplot as plt
from scipy.ndimage import label, generate_binary_structure
import spectral as spy
import hsi_shoot_locs as shoots


def read_img(fn):
    """
    read HSI image and convert to a numpy array
    """
    # Open and read the ENVI file
    image = spy.open_image(fn + ".hdr") 
    image_data = image.load()
    # Convert to numpy array and transpose to form (band, x, y)
    data = np.array(np.transpose(image_data, (2, 0, 1)))
    return data

def get_wav(fn):
    """
    returns both wavlengths and fwhm
    """
    img = spy.open_image(fn + ".hdr")
    wav = img.bands.centers
    fwhm = img.bands.bandwidths
    return wav,fwhm

def get_integration_time(fn):
    img = spy.open_image(fn + ".hdr")
    itime = img.metadata['senop integration time']
    return np.array(itime).astype(float)

def get_gain(fn):
    img = spy.open_image(fn + ".hdr")
    gain = img.metadata['data gain values']
    return np.array(gain).astype(float)

def find_nearest_idx(array, value):
    array = np.asarray(array)
    return  (np.abs(array - value)).argmin()

def calc_ndvi(img,wav):
    i760 = find_nearest_idx(wav, 760)
    i660 = find_nearest_idx(wav, 660)
    ndvi = (img[i760] - img[i660]) / (img[i760] + img[i660])
 
    return ndvi

def calc_pri(img,wav):
    i531 =  find_nearest_idx(wav, 531)
    i570 =  find_nearest_idx(wav, 570)
    pri = (img[i531] - img[i570]) / (img[i531] + img[i570])
    
    return pri

def calc_pri_564(img,wav):
    i531 =  find_nearest_idx(wav, 531)
    i570 =  find_nearest_idx(wav, 564)
    pri = (img[i531] - img[i570]) / (img[i531] + img[i570])
    return pri

def calc_mtci(img,wav): 
    i754 =  find_nearest_idx(wav, 754)
    i709 =  find_nearest_idx(wav, 709)
    i681 =  find_nearest_idx(wav, 681)
    mtci = (img[i754] - img[i709]) / (img[i709] - img[i681])

    return mtci

def calc_rep(img,wav):
    """
    function to find REP using linear four-point interpolation
    """
    i700 = find_nearest_idx(wav, 700)
    i740 = find_nearest_idx(wav, 740)
    i670 = find_nearest_idx(wav, 670)
    i780 = find_nearest_idx(wav, 780)
    ri = (img[i670] + img[i780]) / 2 
    rep = 700 + (40*((ri-img[i700])/(img[i740]-img[i700])))
    return rep

def cohend(sample1, sample2):
    """
    function to calculate Cohen's d for independent samples
    """
    n1, n2 = len(sample1), len(sample2)
    # variance for sample1, sample2
    s1, s2 = np.nanvar(sample1, ddof=1), np.nanvar(sample2, ddof=1)
    # pooled standard deviation
    s = math.sqrt(((n1 - 1) * s1 + (n2 - 1) * s2) / (n1 + n2 - 2))
    # means of the samples
    u1, u2 = np.nanmean(sample1), np.nanmean(sample2)
    # effect size
    return abs((u1 - u2) / s)

def plot_rgb(img, wav, save, outputnm):
    """
    function to plot an RGB image
    """
    iRed = find_nearest_idx(wav, 630)
    iGreen = find_nearest_idx(wav, 530)
    iBlue = 3 # 3rd band (510 nm) is shortest ok band

    rgb_img = np.transpose(img, (1,2,0))

    plt.figure(figsize=(5,5))
    spy.imshow(rgb_img, (iRed, iGreen, iBlue), stretch = 0.01, fignum = 1)
    plt.title(outputnm)

    if save: plt.savefig(outputnm)

    plt.show()

def sample_ave(IT, HSI_notes, rm_sat=True):
    """
    Function to open images with the same IT, remove over exposure, apply gain, and get mean 
    """
    filtered_notes = HSI_notes[(HSI_notes['IT'] == IT) & (HSI_notes['sample'] == 'sample')]
    filtered_notes = filtered_notes.reset_index(drop=True)
    sample_paths = filtered_notes['path']
    gain = get_gain(sample_paths[0])

    arrays = [0] * len(sample_paths)

    for i in range(len(sample_paths)):
        arrays[i] = read_img(sample_paths[i])

    for array in arrays:
        if rm_sat:
            array[array >= (65472 - 64)] = np.nan
        for nband in range(array.shape[0]):
            array[nband] = array[nband]*gain[nband] 

    return np.nanmean(arrays, axis = 0) 

def find_WR_mask(radius, dt):
    center = shoots.wr_locs[str(dt)]
    h = 1024
    w = 1024
    Y, X = np.ogrid[:h, :w]
    dist_from_center = np.sqrt((X - center[0])**2 + (Y-center[1])**2)

    mask = dist_from_center <= radius
    return mask

def wr_spec(arr, dt):
    num_bands = arr.shape[0]
    avg_values = np.zeros(num_bands)

    for band in range(num_bands):
        band_slice = arr[band]
        mask_arr = find_WR_mask(10, dt)
        avg_values[band] = np.nanmean(band_slice[mask_arr])

    WR_array = np.empty(arr.shape)

    for i in range(WR_array.shape[0]):
        fill_val = avg_values[i]
        WR_array[i].fill(fill_val)

    return WR_array

def wr_spec_w_report(arr, output_filename, dt):
    num_bands = arr.shape[0]
    avg_values = np.zeros(num_bands)

    num_cols_subplot = 4
    num_rows_subplot = (num_bands + num_cols_subplot - 1) // num_cols_subplot

    figsize = (15, 4* num_rows_subplot)

    fig, axes = plt.subplots(num_rows_subplot, num_cols_subplot, figsize=figsize)


    for band in range(num_bands):

        band_slice = arr[band]
        mask_arr = find_WR_mask(10, dt)

        avg_values[band] = np.nanmean(band_slice[mask_arr])

        # PLOT all bands with mask
        row_idx = band // num_cols_subplot
        col_idx = band % num_cols_subplot

        masked_array = np.ma.masked_array(band_slice, mask=mask_arr)

        axes[row_idx, col_idx].imshow(masked_array)
        axes[row_idx, col_idx].set_title(f"Layer {band+1}")


    plt.savefig(output_filename)
    plt.show()
    plt.close()

    avg_values[np.isnan(avg_values)] = np.nan

    plt.plot(avg_values)
    plt.title('WR, ' + str(dt))
    plt.savefig(output_filename + '_lineplt')
    plt.close()

    WR_array = np.empty(arr.shape)

    for i in range(WR_array.shape[0]):
        fill_val = avg_values[i]
        WR_array[i].fill(fill_val)

    return WR_array

def get_DC(IT, HSI_notes):
    """
    function to open DC, apply gain, calculate DC for VIS and NIR range, and return a 3d array
    """
    filtered_notes = HSI_notes[(HSI_notes['IT'] == IT) & (HSI_notes['sample'] == 'dark')]
    filtered_notes = filtered_notes.reset_index(drop=True)
    sample_paths = filtered_notes['path']
    gain = get_gain(sample_paths[0])

    arrays = [0] * len(sample_paths)

    for i in range(len(sample_paths)):
        arrays[i] = read_img(sample_paths[i])

    for array in arrays:
        for nband in range(array.shape[0]):
            array[nband] = array[nband]*gain[nband] 

    if not arrays:
        raise ValueError("No matching cubes found") 
    
    DC_av = np.nanmean(arrays, axis=0) 

    DC_array = np.empty(DC_av.shape)

    for i in range(DC_array.shape[0]):
        fill_val = np.nanmean(DC_av[i])
        DC_array[i].fill(fill_val)

    return DC_array


def save_cube(img, header_fn, output_nm):
    file = output_nm + '.hdr'
    hdr_file = os.path.join(os.getcwd(), file)
    
    wav, fwhm = get_wav(header_fn) 
    ittime = get_integration_time(header_fn)

    md = {  'wavelength units': 'nm',
            'lines': img.shape[2],
            'samples': img.shape[1],
            'bands': img.shape[0],
            'data type': 4,
            'wavelength': wav,
            'fwhm': fwhm,
            }
    
    spy.envi.save_image(hdr_file, np.transpose(img, (1, 2, 0)), metadata = md, ext = '.dat')




